<link href='//fonts.googleapis.com/css?family=Oswald:700|Inconsolata:400,700'
          rel='stylesheet' type='text/css'>
<div class="sidebarlogo">
  <a href="{{ pathto('index') }}">
  <div class="title">Attic</div>
</a>
</div>